"""Contains pipelines to transform input."""
from sklearn.compose import ColumnTransformer
from sklearn.ensemble import RandomForestClassifier
from sklearn.impute import SimpleImputer
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import OneHotEncoder, StandardScaler

from src.config.settings import config

num_pipe = Pipeline(
    [("num_impute_median", SimpleImputer(strategy="median")), ("ss", StandardScaler())]
)

cat_pipe = Pipeline([("ohe", OneHotEncoder(drop="first"))])

transform_pipe = ColumnTransformer(
    [
        ("num_transform", num_pipe, config.modelConfig.total_num_features),
        ("cat_transform", cat_pipe, config.modelConfig.cat_features),
    ]
)

survive_pipe_rfc = Pipeline(
    [
        ("transform", transform_pipe),
        ("rfc", RandomForestClassifier(random_state=config.modelConfig.random_state)),
    ]
)



"""Contains function to return prediction of input."""
from src.config.settings import PIPELINE_PATH, config
from src.preprocessing import datamanager, validation


def make_prediction_inputs(input_data: list, proba=False) -> int:
    """
    Load pipeline and preprocess input then predict.

    Parameters
    ----------
    input_data: List
        Input data from streamlit application, expects 1 row of observation
    proba: Bol
        True if return probability instead
    Returns
    -------
    Int | Float
        Int: If proba is not set to True. Returns class of survival. 0 for no, 1 for yes
        Float: If proba is set to True, the probability of survival [0,1]
    """
    survive_pipeline = datamanager.load_pipeline(PIPELINE_PATH)
    processed_input = datamanager.preprocess_input(input_data)

    if proba:
        prediction_proba = survive_pipeline.predict_proba(processed_input)
        return prediction_proba
    else:
        prediction = survive_pipeline.predict(processed_input)
        return prediction


def make_prediction_inputs_api(input_data: list, proba=False) -> dict:
    """
    Load pipeline and preprocess input from API then predict.
    The inputs will be similar to what is in the original training set with
    original features

    Parameters
    ----------
    input_data: List
        Input data from API
    proba: Bol
        True if return probability instead
    Returns
    -------
    Int | Float
        Int: If proba is not set to True. Returns class of survival. 0 for no, 1 for yes
        Float: If proba is set to True, the probability of survival [0,1]
    """
    survive_pipeline = datamanager.load_pipeline(PIPELINE_PATH)
    processed_input = datamanager.preprocess_data(input_data)

    validated_input, errors = validation.api_input_validation(processed_input)

    results = {
        "Prediction": None,
        "Version": config.appConfig.model_version,
        "Errors": errors,
    }

    if errors is not None:
        if proba:
            prediction_proba = survive_pipeline.predict_proba(validated_input)
            results["Prediction"] = prediction_proba
        else:
            prediction = survive_pipeline.predict(validated_input)
            results["Prediction"] = prediction

    return results



#! /usr/bin/env python
"""Runs the application on streamlit."""

import streamlit as st

from src.config.settings import config
from src.model.predict import make_prediction_inputs


def run_streamlit():
    """
    Establish the streamlit UI when run.

    :return: None
    """
    st.title("Predicting Survival of coronary artery disease")

    st.write("Choose inputs below")

    st_gender = str(st.radio("Gender", ("Male", "Female")))
    st_smoker = str(st.radio("Smoker", ("Yes", "No")))
    st_diabetes = str(
        st.radio("Diabetic Condition?", ("Normal", "Pre-diabetes", "Diabetes"))
    )
    st_age = float(st.number_input("Age", value=config.streamlitConfig.age))
    st_ejection_fraction = str(st.radio("Ejection Fraction", ("Low", "Normal-High")))
    st_sodium = float(
        st.number_input("Sodium (mg/dL)", value=config.streamlitConfig.sodium)
    )
    st_creatinine = float(
        st.number_input("Creatinine (md/dL)", value=config.streamlitConfig.creatinine)
    )
    st_pletelets = int(
        st.number_input(
            "Pletelets (kilo-platelets/mL)", value=config.streamlitConfig.pletelets
        )
    )
    st_ck = int(
        st.number_input(
            "Creatinine Phosphokinase (mcg/L)", value=config.streamlitConfig.ck
        )
    )
    st_bp = int(
        st.number_input("Blood pressure(mmHG)", value=config.streamlitConfig.bp)
    )
    st_hemo = float(
        st.number_input("Hemoglobin (g/dL)", value=config.streamlitConfig.hemo)
    )
    st_height = int(
        st.number_input("Height in cm", value=config.streamlitConfig.height)
    )
    st_weight = int(
        st.number_input("Weight in Kg", value=config.streamlitConfig.weight)
    )

    inputs = [
        st_gender,
        st_smoker,
        st_diabetes,
        st_age,
        st_ejection_fraction,
        st_sodium,
        st_creatinine,
        st_pletelets,
        st_ck,
        st_bp,
        st_hemo,
        st_height,
        st_weight,
    ]

    # st.write(config.modelConfig.total_features)

    if st.button("Predict"):

        bmi = (st_weight / st_height / st_height) * 10000
        inputs.append(bmi)
        prediction = make_prediction_inputs(inputs)
        predict_proba = make_prediction_inputs(inputs, proba=True)
        if prediction == 0:
            st.write(
                "Prediction is "
                + strip_brackets(prediction)
                + " with probability "
                + strip_brackets(predict_proba[0][prediction])
            )
            st.write("Please see a doctor!")
        else:
            st.write(
                "Prediction is "
                + strip_brackets(prediction)
                + " with probability "
                + strip_brackets(predict_proba[0][prediction])
            )
            st.write("Please keep up the healthy habits")


def strip_brackets(_string: str) -> str:
    """
    Strip the brackets [ ] out of strings and returns it without brackets.

    :param _string: a string with brackets
    :Returns: string without brackets
    """
    return str(_string).replace("[", "").replace("]", "")


if __name__ == "__main__":
    run_streamlit()



"""test the bound outliers function."""
import pandas as pd

from src.config.settings import SAMPLE_DATA_PATH, config
from src.preprocessing.datamanager import bound_outliers, preprocess_data

sample_data = pd.read_csv(SAMPLE_DATA_PATH)
processed_sample_data = preprocess_data(sample_data)


def test_bound_outliers():
    """
    Test if the outliers have been cap to the min or max.

    :return: None
    """
    arbitary_value = 1
    for col in config.modelConfig.original_num_features:
        ori_min = processed_sample_data[col].min()
        ori_max = processed_sample_data[col].max()

        bound_outliers(processed_sample_data, col)

        current_min = processed_sample_data[col].min()
        current_max = processed_sample_data[col].max()

        assert ori_min - arbitary_value <= current_min
        assert ori_max + arbitary_value >= current_max



"""Manages all of the data processing."""
import sqlite3 as sq3

import joblib
import numpy as np
import pandas as pd

from src.config.settings import config


def preprocess_input(inputs: list) -> pd.DataFrame:
    """
    Preprocess inputs from streamlit or list of observations. It will return a pandas dataframe with the columns.

    :param inputs: list of input conforming with the columns of the database
    :return: Pandas dataframe of the list
    """
    inputs = np.array(inputs).reshape(1, -1)
    rename_cols = {
        k: v
        for k, v in zip(
            range(len(config.modelConfig.total_features)),
            config.modelConfig.total_features,
        )
    }
    df = pd.DataFrame(inputs).rename(columns=rename_cols)
    return df


def preprocess_data(df: pd.DataFrame) -> pd.DataFrame:
    """
    Preprocess inputs from database file.

    :param df: list of input conforming with the columns of the database
    :return: Processed pandas dataframe
    """
    df.drop(columns=["ID", "Favorite color"], inplace=True)

    bound_numerical_features(df)

    df.Smoke = df.Smoke.replace("NO", "No")
    df.Smoke = df.Smoke.replace("YES", "Yes")
    df.Age = np.where(df.Age < 0, -df.Age, df.Age)
    df["Ejection Fraction"] = (
        df["Ejection Fraction"].replace("L", "Low").replace("N", "Normal")
    )
    df["Ejection Fraction"] = (
        df["Ejection Fraction"]
        .replace("High", "Normal")
        .replace("Normal", "Normal-High")
    )

    df.Survive = df.Survive.str.replace("No", "0")
    df.Survive = df.Survive.str.replace("Yes", "1")
    df.Survive = df.Survive.astype("int")
    df["BMI"] = (df.Weight / df.Height / df.Height) * 10000

    return df


def load_from_database(db_path: str) -> pd.DataFrame:
    """
    Load up the database into a pandas dataframe and returns it.

    :param db_path: str containing the path of the database
    :return: pandas dataframe
    """
    con = sq3.Connection(db_path)

    query = """SELECT * FROM SURVIVE"""
    df: pd.DataFrame = pd.read_sql(query, con)

    df_processed = preprocess_data(df)
    return df_processed


def load_pipeline(pipe_path: str):
    """
    Load the pipeline from a path.

    :param pipe_path:str containing the path to the pipeline
    :return: the pipeline
    """
    pipe = joblib.load(filename=pipe_path)
    return pipe


def return_min_max_boxplot(df, col, min_or_max):
    """
    Return upper(3rd quartile-1.5 of inter-quartile range) or lower(1rd quartile+1.5 of inter-quartile range).

    :param df : dataframe
    :param col: the col of the dataframe
    :param min_or_max : the upper(max) or lower(min) of the inner fence boxplot value
    :return: (float) upper(max) or lower(min) of the inner fence boxplot value
    """
    df_col_q1 = np.quantile(df[col], 0.25, method="midpoint")
    df_col_q3 = np.quantile(df[col], 0.75, method="midpoint")
    df_col_itr = df_col_q3 - df_col_q1
    df_col_min_cap = df_col_q1 - (1.5 * df_col_itr)
    df_col_max_cap = df_col_q3 + (1.5 * df_col_itr)
    if "min" in min_or_max:
        return df_col_min_cap
    else:
        return df_col_max_cap


def bound_outliers(df: pd.DataFrame, col: str):
    """
    Squeezes outliers to within the boxplot inner fences.

    :param df: dataframe
    :param col: the col of the dataframe
    :return: None
    """
    df_col_max = return_min_max_boxplot(df, col, "max")
    df_col_min = return_min_max_boxplot(df, col, "min")
    df[col] = np.where(df[col] >= df_col_max, df_col_max, df[col])
    df[col] = np.where(df[col] <= df_col_min, df_col_min, df[col])


def bound_numerical_features(df: pd.DataFrame):
    """
    Bounds any outliers found in numerical features of a pandas dataframe in the box plot distribution.

    :param df: pd.DataFrame Numerical dataframe
    :return: None
    """
    num_cols = df.select_dtypes(include=["int", "float"]).columns.to_list()
    for col in num_cols:
        bound_outliers(df, col)



"""Tests the predict function in predict.py."""
from src.model.predict import make_prediction_inputs
from src.preprocessing.datamanager import preprocess_input

TOTAL_FEATURES = [
    "Gender",
    "Smoke",
    "Diabetes",
    "Age",
    "Ejection Fraction",
    "Sodium",
    "Creatinine",
    "Pletelets",
    "Creatinine phosphokinase",
    "Blood Pressure",
    "Hemoglobin",
    "Height",
    "Weight",
    "BMI",
]

# sample input with no smoking and baseline CK
sample_input_0 = [
    "Male",
    "No",  # smoking
    "Normal",
    75,
    "Low",
    134,
    2.5,
    224000,
    350,  # CK
    162,
    13,
    140,
    47,
    23.97959183673469,
]

# sample input with smoking
sample_input_1 = [
    "Male",
    "Yes",  # smoking
    "Normal",
    75,
    "Low",
    134,
    2.5,
    224000,
    350,
    162,
    13,
    140,
    47,
    23.97959183673469,
]

# sample input with increased CK
sample_input_2 = [
    "Male",
    "No",  # smoking
    "Normal",
    75,
    "Low",
    134,
    2.5,
    224000,
    550,  # CK
    162,
    13,
    140,
    47,
    23.97959183673469,
]


def test_predict():
    """
    Post train test where known factors (smoking/ CK) should affect survival rate negatively.

    :return: None
    """
    processed_sample_input_0 = preprocess_input(sample_input_0)
    processed_sample_input_1 = preprocess_input(sample_input_1)
    processed_sample_input_2 = preprocess_input(sample_input_2)

    pred_proba_processed_sample_input_0 = make_prediction_inputs(
        processed_sample_input_0, proba=True
    )
    pred_proba_processed_sample_input_1 = make_prediction_inputs(
        processed_sample_input_1, proba=True
    )
    pred_proba_processed_sample_input_2 = make_prediction_inputs(
        processed_sample_input_2, proba=True
    )

    # survival rate for sample input 0 should be higher than sample input 1
    assert (
        pred_proba_processed_sample_input_0[0][1]
        > pred_proba_processed_sample_input_1[0][1]
    )

    # survival rate for sample input 0 should be higher than sample input 2
    assert (
        pred_proba_processed_sample_input_0[0][1]
        > pred_proba_processed_sample_input_2[0][1]
    )



"""Tests the load from database function."""
import pandas as pd

from src.config.settings import DATABASE_PATH, SAMPLE_DATA_PATH, config
from src.preprocessing.datamanager import load_from_database, preprocess_data

sample_data = pd.read_csv(SAMPLE_DATA_PATH)
processed_sample_data = preprocess_data(sample_data)

sample_input = [
    "Male",
    "Yes",
    "Normal",
    21,
    "Low",
    120,
    0.2,
    266000,
    100,
    105,
    16.3,
    185,
    80,
    23.374726077428782,
]


def test_load_from_database():
    """
    Checks if the columns retrieved are all present.

    :return: None
    """
    sample_database = load_from_database(DATABASE_PATH)

    assert (
        sample_database.columns.to_list()
        == config.modelConfig.total_features_with_target
    )



"""tests the pipeline function."""
import numpy as np
import pandas as pd

from src.config.settings import SAMPLE_DATA_PATH
from src.model.pipeline import transform_pipe
from src.preprocessing.datamanager import preprocess_data


def test_pipeline():
    """
    Test no Na values from pipeline output.

    :return: None
    """
    sample_data = pd.read_csv(SAMPLE_DATA_PATH)
    processed_sample_data = preprocess_data(sample_data)

    transformed_processed_sample_data = transform_pipe.fit_transform(
        processed_sample_data
    )

    assert np.all(pd.DataFrame(transformed_processed_sample_data).isna()).sum() == 0



"""tests preprocess input from preprocessing datamanager function."""
import numpy as np
import pandas as pd

from src.config.settings import SAMPLE_DATA_PATH, config
from src.preprocessing.datamanager import preprocess_data, preprocess_input

sample_data = pd.read_csv(SAMPLE_DATA_PATH)
processed_sample_data = preprocess_data(sample_data)

sample_input = [
    "Male",
    "Yes",
    "Normal",
    21,
    "Low",
    120,
    0.2,
    266000,
    100,
    105,
    16.3,
    185,
    80,
    23.374726077428782,
]


def test_preprocess_input():
    """
    Test if the processed input has the correct cols, data types, shape.

    :return: None
    """
    processed_inputs = preprocess_input(sample_input)
    assert processed_inputs.columns.to_list() == config.modelConfig.total_features
    assert processed_inputs.shape[1] == len(config.modelConfig.total_features)
    for feature in config.modelConfig.cat_features:
        assert processed_inputs[feature].dtypes is np.dtype(object)

    for feature in config.modelConfig.total_num_features:
        assert processed_inputs[feature].dtypes is float or int



from typing import List, Optional

import pandas as pd
from pydantic import BaseModel, ValidationError


class inputSchema(BaseModel):

    Survive: str
    Gender: str
    Smoke: str
    Diabetes: str
    Age: int
    Ejection_Fraction: str  # renamed
    Sodium: int
    Creatinine: float
    Pletelets: int
    CK: int  # renamed
    BP: int  # renamed
    Hemoglobin: float
    Height: int
    Weight: int
    BMI: float  # new feature


class multiple_inputSchema(BaseModel):
    inputs: List[inputSchema]


def api_input_validation(input_data: pd.DataFrame) -> [pd.DataFrame, Optional[dict]]:
    try:
        multiple_inputSchema(inputs=input_data)

    except ValidationError as error:
        errors = error.json()

    return input_data, errors



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path

from setuptools import find_packages, setup

# Package meta-data.
NAME = "heart disease classification model"
DESCRIPTION = "Classifying whether the patient will survive"
URL = "https://github.com/ChngYuanLongRandy/demo_deploy"
EMAIL = "chngyuanlong@gmail.com"
AUTHOR = "RandyChng"
REQUIRES_PYTHON = "==3.10.0"
VERSION = "0.0.2"


# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the
# Trove Classifier for that!
with open("README.md", "r") as f:
    long_description = f.read()


def list_reqs(fname="requirements.txt"):
    with open(fname) as fd:
        return fd.read().splitlines()


# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=("tests",)),
    # package_data={"regression_model": ["VERSION"]},
    install_requires=list_reqs(),
    # extras_require={},
    include_package_data=True,
    license="BSD-3",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
    ],
)



"""test the preprocess data function from preprocessing datamanager.py."""
import pandas as pd

from src.config.settings import SAMPLE_DATA_PATH
from src.preprocessing.datamanager import preprocess_data

sample_input = [
    "Male",
    "Yes",
    "Normal",
    21,
    "Low",
    120,
    0.2,
    266000,
    100,
    105,
    16.3,
    185,
    80,
    23.374726077428782,
]


def test_preprocess_data():
    """
    Test if the processed data has the correct cols, data types, shape and unique values.

    :return: None
    """
    sample_data = pd.read_csv(SAMPLE_DATA_PATH)
    processed_sample_data = preprocess_data(sample_data)
    # Check columns
    assert "ID" not in processed_sample_data.columns.to_list()
    assert "Favourite Color" not in processed_sample_data.columns.to_list()

    # check unique values
    assert len(processed_sample_data["Smoke"].value_counts()) == 2
    assert len(processed_sample_data["Survive"].value_counts()) == 2
    assert len(processed_sample_data["Ejection Fraction"].value_counts()) == 2

    pass



"""Contains all of the constant variables used."""
import os
from pathlib import Path
from typing import List

from pydantic import BaseModel
from strictyaml import YAML, load

# main folders
CONFIG_PATH = os.path.dirname(__file__)  # aiap/src/config
SRC_ROOT = Path(CONFIG_PATH).parent  # aiap/src
ROOT = Path(SRC_ROOT).parent  # aiap

DATA_PATH = os.path.join(ROOT, "data")  # aiap/data
MODEL_PATH = os.path.join(SRC_ROOT, "model")  # aiap/src/model
PREPROCESSING_PATH = os.path.join(SRC_ROOT, "preprocessing")  # aiap/src/preprocessing
CONFIG_FILE_PATH = os.path.join(ROOT, "config.yml")  # aiap/config.yml

# Objects
DATABASE_PATH = os.path.join(DATA_PATH, "survive.db")  # aiap/data/survive.db
SAMPLE_DATA_PATH = os.path.join(DATA_PATH, "sample_df.csv")  # aiap/data/sample_df.csv
PIPELINE_NAME = "pipeline.pkl"
PIPELINE_PATH = os.path.join(MODEL_PATH, PIPELINE_NAME)  # aiap/src/model/pipeline.pkl
LOG_OUTPUT_NAME = "log_file.txt"
LOG_OUTPUT_PATH = os.path.join(
    MODEL_PATH, LOG_OUTPUT_NAME
)  # aiap/src/model/log_file.txt

# model specific objects
PARAMS = {}  # insert params configuration here


class AppConfig(BaseModel):
    """
    Application-level config
    """

    package_name: str
    pipeline_save_file: str
    model_version: str


class ModelConfig(BaseModel):
    """
    Model-level config
    """

    # features....
    target: str
    features: List[str]
    original_features: List[str]
    renamed_features: List[str]
    cat_features: List[str]
    original_num_features: List[str]
    total_features: list[str]
    total_num_features: list[str]
    total_features_with_target: List[str]

    # model...
    model_name: str
    test_size: float
    random_state: int
    cv: int
    random_state: int
    model_name: str


class StreamlitConfig(BaseModel):
    age: int
    sodium: int
    creatinine: float
    pletelets: int
    ck: int
    bp: int
    hemo: float
    height: int
    weight: int


class mainConfig(BaseModel):
    appConfig: AppConfig
    modelConfig: ModelConfig
    streamlitConfig: StreamlitConfig


def locate_config_file() -> Path:
    """
    locates the config.yml file. If its missing will raise an error
    :return: Path - Path of the config file
    """
    if Path(CONFIG_FILE_PATH).is_file():
        return CONFIG_FILE_PATH
    else:
        raise Exception(f"Config file is not found at {CONFIG_FILE_PATH}")


def load_config(cfg_path: Path = None) -> YAML:
    """
    Loads the config from the default path unless specified
    :param cfg_path: Path object that specifies path of config.yml file
    :return: YAML file
    """
    # default file
    if cfg_path is None:
        cfg_path = locate_config_file()

    # custom config.yml file specified
    if cfg_path:
        try:
            with open(cfg_path, "r") as f:
                parsed_config = load(f.read())
                return parsed_config
        except OSError:
            raise OSError(
                f"Did not manage to find the config.yml file specified at {cfg_path}"
            )


def create_and_validate_config(parsed_config: YAML = None) -> mainConfig:
    """
    using the YAML file it will instantiate a config file
    :param parsed_config: YAML file from config.yml
    :return: mainConfig object
    """
    if parsed_config is None:
        parsed_config = load_config()

    _config = mainConfig(
        appConfig=AppConfig(**parsed_config.data),
        modelConfig=ModelConfig(**parsed_config.data),
        streamlitConfig=StreamlitConfig(**parsed_config.data),
    )

    return _config


config = create_and_validate_config()



"""Contains functions regarding training of pipeline."""

from time import time

import joblib
from sklearn.metrics import classification_report, roc_auc_score
from sklearn.model_selection import GridSearchCV, cross_validate, train_test_split

from src.config.settings import (
    DATABASE_PATH,
    LOG_OUTPUT_PATH,
    PARAMS,
    PIPELINE_PATH,
    config,
)
from src.model.pipeline import survive_pipe_rfc
from src.preprocessing import datamanager


def run_trainpipeline(gridsearch=False):
    """
    Trains then saves the pipeline and outputs training log with scoring metrics.Allows gridsearch to be done.

    :param gridsearch: Bol True to run gridsearch on pipeline
    :return: None
    """
    df = datamanager.load_from_database(DATABASE_PATH)

    X_train, X_test, y_train, y_test = train_test_split(
        df[config.modelConfig.total_features],
        df[config.modelConfig.target],
        test_size=config.modelConfig.test_size,
        random_state=config.modelConfig.random_state,
    )

    if gridsearch:
        print("Gridsearch = True")
        try:
            model = GridSearchCV(survive_pipe_rfc, PARAMS, cv=config.modelConfig.cv)
            score_pipeline(model, X_train, y_train, X_test, y_test)
            joblib.dump(model, PIPELINE_PATH)
        except Exception:
            raise Exception("Unable to proceed, please check Parameters")

        joblib.dump(model, PIPELINE_PATH)
    else:
        print("Gridsearch = False")
        survive_pipe_rfc.fit(X_train, y_train)
        score_pipeline(survive_pipe_rfc, X_train, y_train, X_test, y_test)
        joblib.dump(survive_pipe_rfc, PIPELINE_PATH)


def score_pipeline(model, X_train, y_train, X_test, y_test):
    """
    Scores the pipeline  with F1, recall, precision, ROC-AUC and outputs.

    :param model: pipeline
    :param X_train: X Training set
    :param y_train: Y Training set
    :param X_test: X test set
    :param y_test: Y test set
    :return: None
    """
    now = time()
    y_pred = model.predict(X_test)

    model_cv_score = cross_validate(
        model,
        X_train,
        y_train,
        cv=config.modelConfig.cv,
        n_jobs=-1,
        return_train_score=True,
        scoring="accuracy",
    )
    then = time()
    diff = then - now
    output_logfile(model, model_cv_score, y_test, y_pred, diff)


def output_logfile(model, model_cv_score, y_test, y_pred, diff):
    """
    Ouput the training log on a output file with scoring metrics to LOG_OUTPUT_PATH.

    :param model: pipeline
    :param model_cv_score: Cross validation score
    :param y_test: y test set
    :param y_pred: prediction for y_test for scoring
    :param diff: How long it took the model to run
    :return:
    """
    with open(LOG_OUTPUT_PATH, "w") as log_file:
        log_file.write("Log file for model results\n")
        log_file.write("Model is " + config.modelConfig.model_name + "\n")
        log_file.write(
            f"Classifcation report :\n{classification_report(y_test, y_pred)}" + "\n"
        )
        log_file.write(
            f'Average CV train acc score : \n {model_cv_score["train_score"].mean()} '
            + "\n"
        )
        log_file.write(
            f'Average CV test acc score : \n {model_cv_score["test_score"].mean()} '
            + "\n"
        )
        log_file.write(f"ROC AUC score :\n {roc_auc_score(y_test, y_pred)}" + "\n")
        log_file.write(f"Seconds taken to run results is {diff}")


if __name__ == "__main__":
    run_trainpipeline()
